'use strict';

var fs = require('fs');
var path = require('path');

/**
 * @description 是否是文件夹
 * @param path
 * @returns
 */
const isDir = (path) => {
    return fs.lstatSync(path).isDirectory();
};
/**
 * @description 是否是文件
 * @param path
 * @returns
 */
const isFile = (path) => {
    return fs.lstatSync(path).isFile();
};
/**
 * @description 获取所有文件夹
 * @param path
 * @returns
 */
const getFiles = (path$1) => {
    const dirs = fs.readdirSync(path$1);
    return dirs.reduce((result, name) => {
        const fullPath = path.join(path$1, name);
        isFile(fullPath) && result.push({ fileName: name, filePath: fullPath });
        return result;
    }, []);
};
/**
 * @description 合并配置
 * @param source object
 * @param target object
 * @returns object
 */
function mergeConfig(source, target) {
    const isObject = (data) => Object.prototype.toString.call(data) === '[object Object]';
    const hasKey = (data, key) => isObject(data) && Reflect.has(data, key);
    for (const key in target) {
        if (!hasKey(source, key)) {
            source[key] = target[key];
            isObject(target[key]) && mergeConfig(source[key], target[key]);
        }
        else {
            isObject(target[key]) && mergeConfig(source[key], target[key]);
        }
    }
    return source;
}
/**
 * @description 分离样式
 * @param files GetFiles
 * @param suffixs Array<string>
 */
function extract(files, suffixs) {
    return suffixs.reduce((result, suffix) => {
        result[suffix] = files.filter(item => item.fileName.endsWith(`.${suffix}`));
        return result;
    }, {});
}

/**
 * @description 插件名称
 */
const PLUGIN_NAME = 'vue-cli-plugin-inject-style';
/**
 * @description 样式配置
 */
const styleConfig = {
    css: {
        loaderOptions: {
            scss: {
                additionalData: ''
            },
            less: {
                additionalData: ''
            }
        }
    }
};
/**
 * @description 处理插件默认配置
 * @param root string
 */
const getPluginConfig = (root) => {
    return {
        [PLUGIN_NAME]: {
            path: path.join(root, 'src', 'style'),
            suffixs: ['scss', 'less']
        }
    };
};

function injectStyle(api, options) {
    const { css = {}, pluginOptions = {} } = options;
    const root = api.getCwd();
    // 插件配置
    const pluginConfig = mergeConfig(pluginOptions, getPluginConfig(root))[PLUGIN_NAME];
    if (!isDir(pluginConfig.path))
        return;
    // 所有的样式文件路径
    const files = getFiles(pluginConfig.path);
    const extractStyle = extract(files, pluginConfig.suffixs);
    const cssOptions = mergeConfig(css, styleConfig.css);
    Object.keys(extractStyle).forEach(suffix => {
        const additionalData = cssOptions.loaderOptions[suffix].additionalData ?? '';
        cssOptions.loaderOptions[suffix].additionalData =
            extractStyle[suffix].map(item => `@import "${item.filePath}";`).join('') + `${additionalData}`;
    });
    options.css = cssOptions;
}

module.exports = injectStyle;
